"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function slotHandler(documentation, templateAst, options) {
    if (templateAst.tag === 'slot') {
        var bindings = extractAndFilterAttr(templateAst.attrsMap);
        var name_1 = 'default';
        if (bindings.name) {
            name_1 = bindings.name;
            delete bindings.name;
        }
        var slotDescriptor = documentation.getSlotDescriptor(name_1);
        slotDescriptor.bindings = bindings;
        var comment = '';
        if (templateAst.parent) {
            var slotSiblings = templateAst.parent.children;
            // First find the position of the slot in the list
            var i = slotSiblings.length - 1;
            var currentSlotIndex = -1;
            do {
                if (slotSiblings[i] === templateAst) {
                    currentSlotIndex = i;
                }
            } while (currentSlotIndex < 0 && i--);
            // Find the first leading comment node as a description of the slot
            var slotSiblingsBeforeSlot = slotSiblings.slice(0, currentSlotIndex).reverse();
            for (var _i = 0, slotSiblingsBeforeSlot_1 = slotSiblingsBeforeSlot; _i < slotSiblingsBeforeSlot_1.length; _i++) {
                var potentialComment = slotSiblingsBeforeSlot_1[_i];
                // if there is text between the slot and the comment ignore
                if (potentialComment.type !== 3 ||
                    (!potentialComment.isComment && potentialComment.text.trim())) {
                    break;
                }
                if (potentialComment.isComment &&
                    !(templateAst.parent.tag === 'slot' && templateAst.parent.children[0] === potentialComment)) {
                    comment = potentialComment.text.trim();
                    break;
                }
            }
        }
        else if (options.rootLeadingComment.length) {
            comment = options.rootLeadingComment;
        }
        if (comment.length && comment.search(/\@slot/) !== -1) {
            slotDescriptor.description = comment.replace('@slot', '').trim();
        }
    }
}
exports.default = slotHandler;
var dirRE = /^(v-|:|@)/;
var allowRE = /^(v-bind|:)/;
function extractAndFilterAttr(attrsMap) {
    var res = {};
    var keys = Object.keys(attrsMap);
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        if (!dirRE.test(key) || allowRE.test(key)) {
            res[key.replace(allowRE, '')] = attrsMap[key];
        }
    }
    return res;
}
//# sourceMappingURL=slotHandler.js.map